#
# es2.s
#
# Settore di avviamento per floppy, stampa una stringa a schermo,
#  carica un modulo (presente sul floppy dopo il primo settore)
#  e lo mette in esecuzione
#

.code16		# Necessario per eseguire il codice in modo reale

.set SEG_BOOT, 0x07c0
.set INIT_STACK, 0x0400

.set SEG_DEST, 0x0800
.set DIM_MODULO, 20		# settori del modulo

	.text

	movw $SEG_BOOT, %ax	# caricamento di valori noti nei registri 
	movw %ax, %ds		#  selettore
	movw %ax, %es

	cli			# interruzioni disabilitate nel cambiamento
				#  di pila
	movw %ax, %ss
	movw $INIT_STACK, %sp	# pila iniziale a 0x07c0:0400, cioe'
				#  512 byte dopo la fine del settore
				#  di avviamento in memoria

	sti

	movw $mess, %si		# indirizzo del primo carattere del messaggio
				#  in si

				# impostazione parametri int 0x10:
	movb $0x0e, %ah		# servizio stampa carattere
	movb $0, %bh		# numero di pagina

nuovo_car:
	lodsb			# carica il carattere in al
	cmpb $0, %al		# verifica se e' l' ultimo
	je fine_str
	int $0x10		# lo stampa
	jmp nuovo_car

fine_str:
	call carica		# carica il modulo da floppy
	call spegni		# spegne il motore del floppy

	ljmp $SEG_DEST, $0	# salto al modulo appena caricato

#
# Caricamento del modulo
# (non si salvano i registri)

da_leggere: .word DIM_MODULO

carica:
	xorw %ax, %ax		# reset del controllore del floppy
	xorw %dx, %dx
	int $0x13

	movw $SEG_DEST, %ax	# in es il segmento di destinazione
	movw %ax, %es
	movw $0x0002, %cx	# la lettura inizia dal secondo settore
				#  del primo cilindro (i settori sono
				#  contati da uno, cilindri e testine da
				#  zero)
	movw $0x0000, %dx	# prima testina del drive zero (primo floppy)

nuovo_sett:
	movw $0x0201, %ax	# lettura di un settore
	xorw %bx, %bx		# azzeramento offset

	int $0x13

	# jc errore_di_lettura	# cf=1 indica un errore in lettura

	decw da_leggere		# decremento del contatore
	cmpw $0, da_leggere
	je fine_car

	movw %es, %ax		# incremento del selettore in es:
	addw $0x20, %ax		#  ogni lettura si avanza di 512 byte, il
	movw %ax, %es		#  nuovo selettore e' dato dal precedente
				#  piu' 512/16 = 32

	incb %cl		# incremento il settore
	cmpb $18, %cl		# se arrivo al 18-esimo devo cambiare testina
	jbe nuovo_sett

	xorb $1, %dh		# cambio di testina
	movb $1, %cl		# riprende a leggere dal primo settore
	jnz nuovo_sett		# xor e' l' ultima istruzione a modificare
				#  eflag, se la testina e' tornata 0
				#  e' necessario cambiare anche cilindro
				#  (non salta)

	incb %ch		# incremento del cilindro
	jmp nuovo_sett

fine_car:
	ret

#
# Spegnimento del motore del floppy tramite il controllore
#

spegni:
	movw $0x3f2, %dx
	movb $0, %al
	outb %al, %dx
	ret

mess:	.asciz "Caricamento in corso"

	.org 510
sign:	.byte	0x55		# signature per un settore di avviamento valido
	.byte	0xaa

