#
# es3.s
#
# Settore di avviamento per floppy, stampa una stringa a schermo,
#  carica un modulo (presente sul floppy dopo il primo settore),
#  abilita la linea A20, abilita il modo protetto e mette in esecuzione
#  il modulo caricato
#

.code16		# Necessario per eseguire il codice in modo reale

.set SEG_BOOT, 0x07c0
.set INIT_STACK, 0x0400

.set SEG_DEST, 0x0800
.set DIM_MODULO, 20		# settori del modulo

.set SEL_CODICE, 8		# selettore segm. codice (modo protetto)
.set SEL_DATI, 0x10		# selettore segm. dati (modo protetto)

	.text

	movw $SEG_BOOT, %ax	# caricamento di valori noti nei registri 
	movw %ax, %ds		#  selettore
	movw %ax, %es

	cli			# interruzioni disabilitate nel cambiamento
				#  di pila
	movw %ax, %ss
	movw $INIT_STACK, %sp	# pila iniziale a 0x07c0:0400, cioe'
				#  512 byte dopo la fine del settore
				#  di avviamento in memoria

	sti

	movw $mess, %si		# indirizzo del primo carattere del messaggio
				#  in si

				# impostazione parametri int 0x10:
	movb $0x0e, %ah		# servizio stampa carattere
	movb $0, %bh		# numero di pagina

nuovo_car:
	lodsb			# carica il carattere in al
	cmpb $0, %al		# verifica se e' l' ultimo
	je fine_str
	int $0x10		# lo stampa
	jmp nuovo_car

fine_str:
	call carica		# carica il modulo da floppy
	call spegni		# spegne il motore del floppy

	cli
	call abilita_A20	# abilitazione della linea A20

	lgdt pd_gdt		# caricamento di GDTR
	movl %cr0, %eax
	andl $0x1fffffff, %eax	# PG=0, CD=0, NW=0, (abilita cache, WB)
	orl $1, %eax		# PE=1 (abilita modo protetto)
	movl %eax, %cr0		# abilitazione del modo protetto

	ljmp $SEL_CODICE, $(go + SEG_BOOT*16)	# salto inter-segment
				# all' istruzione successiva

.code32
go:
	movw $SEL_DATI, %ax	# caricamento dei registri selettore
	movw %ax, %ds
	movw %ax, %es
	movw %ax, %ss
	movl $0x8000, %esp	# nuova pila (prima del modulo caricato)

	xorw %ax, %ax		# registri non usati		
	movw %ax, %fs
	movw %ax, %gs

	lidt pd_idt		# nuova idt

	# sti			# riabilitazione delle interruzioni (non viene
				#  fatta dato che la IDT e' vuota)

	ljmp $SEL_CODICE, $0x8000	# salto al modulo caricato

.code16

#
# Caricamento del modulo
# (non si salvano i registri)
#

da_leggere: .word DIM_MODULO

carica:
	xorw %ax, %ax		# reset del controllore del floppy
	xorw %dx, %dx
	int $0x13

	movw $SEG_DEST, %ax	# in es il segmento di destinazione
	movw %ax, %es
	movw $0x0002, %cx	# la lettura inizia dal secondo settore
				#  del primo cilindro (i settori sono
				#  contati da uno, cilindri e testine da
				#  zero)
	movw $0x0000, %dx	# prima testina del drive zero (primo floppy)

nuovo_sett:
	movw $0x0201, %ax	# lettura di un settore
	xorw %bx, %bx		# azzeramento offset

	int $0x13

	# jc errore_di_lettura	# cf=1 indica un errore in lettura

	decw da_leggere		# decremento del contatore
	cmpw $0, da_leggere
	je fine_car

	movw %es, %ax		# incremento del selettore in es:
	addw $0x20, %ax		#  ogni lettura si avanza di 512 byte, il
	movw %ax, %es		#  nuovo selettore e' dato dal precedente
				#  piu' 512/16 = 32

	incb %cl		# incremento il settore
	cmpb $18, %cl		# se arrivo al 18-esimo devo cambiare testina
	jbe nuovo_sett

	xorb $1, %dh		# cambio di testina
	movb $1, %cl		# riprende a leggere dal primo settore
	jnz nuovo_sett		# xor e' l' ultima istruzione a modificare
				#  eflag, se la testina e' tornata 0
				#  e' necessario cambiare anche cilindro
				#  (non salta)

	incb %ch		# incremento del cilindro
	jmp nuovo_sett

fine_car:
	ret

#
# Spegnimento del motore del floppy tramite il controllore
#

spegni:
	movw $0x3f2, %dx
	movb $0, %al
	outb %al, %dx
	ret

# Abilitazione linea A20
#

abilita_A20:
	call svuota_buffer
	movb $0xd1, %al
	outb %al, $0x64
	call svuota_buffer
	movb $0xdf, %al
	outb %al, $0x60
	call svuota_buffer
	ret

# Svuotamento del buffer della tastiera
#
svuota_buffer:
	jmp 1f
1:	jmp 1f
1:
	inb $0x64, %al
	testb $2, %al
	jne svuota_buffer
	ret

mess:	.asciz "Caricamento in corso"

# pseudo-descrittore della GDT (base = $gdt, limite = 24))
#
pd_gdt: .word 24, 0x7c00 + gdt, 0

# pseudo-descrittore della IDT del modo protetto (vuota)
#
pd_idt:	.word 0, 0, 0

# GDT iniziale
#
gdt:	.word 0, 0, 0, 0			# primo elemento non usato
	# Base=0, Limite=0xfffff, DPL=0, P=1, Tipo=Codice leggibile , G=1
	.word 0xffff, 0x0000, 0x9a00, 0x00cf	# segm. codice (sel. 8)
	# Base=0, Limite=0xfffff, DPL=0, P=1, Tipo=Dati scrivibile , G=1
	.word 0xffff, 0x0000, 0x9200, 0x00cf	# segm. dati (sel. 0x10)

	.org 510
sign:	.byte	0x55		# signature per un settore di avviamento valido
	.byte	0xaa

